@@ -14,6 +14,7 @@ class TestGemPackageTarWriter < Gem::Package::TarTestCase
 
   def teardown
     @tar_writer.close unless @tar_writer.closed?
+    @io.close!
 
     super
   end
@@ -223,26 +224,30 @@ class TestGemPackageTarWriter < Gem::Package::TarTestCase
     name = File.join 'a', 'b' * 100
     assert_equal ['b' * 100, 'a'], @tar_writer.split_name(name)
 
-    assert_raises Gem::Package::TooLongFileName do
-      name = File.join 'a', 'b' * 101
+    name = File.join 'a', 'b' * 101
+    exception = assert_raises Gem::Package::TooLongFileName do
       @tar_writer.split_name name
     end
+    assert_includes exception.message, name
   end
 
   def test_split_name_too_long_prefix
     name = File.join 'a' * 155, 'b'
     assert_equal ['b', 'a' * 155], @tar_writer.split_name(name)
 
-    assert_raises Gem::Package::TooLongFileName do
-      name = File.join 'a' * 156, 'b'
+    name = File.join 'a' * 156, 'b'
+    exception = assert_raises Gem::Package::TooLongFileName do
       @tar_writer.split_name name
     end
+    assert_includes exception.message, name
   end
 
   def test_split_name_too_long_total
-    assert_raises Gem::Package::TooLongFileName do
-      @tar_writer.split_name 'a' * 257
+    name = 'a' * 257
+    exception = assert_raises Gem::Package::TooLongFileName do
+      @tar_writer.split_name name
     end
+    assert_includes exception.message, name
   end
 
 end
